;-------------------------------------------------------------------------
;- FILE:	lib_fir.asm
;-
;- COPYRIGHT	(C) 2011 Atmel Corp. <www.atmel.com>
;-
;- ABSTRACT:	FIR filter and supporting routines
;-
;- DESCRIPTION:	function parameters are assumed to be passed in registers
;-
;- MODIFICATION HISTORY:
;- 2011/05/27 : created	for Keil A51
;-
;-------------------------------------------------------------------------
$GEN


NAME lib_fir

SFR DPCF = 0xA2
SFR DSPR = 0xE2
SFR FIRD = 0xE3
SFR AX   = 0xE1
SFR BX   = 0xF7

SFR DPL0 = 0x82
SFR DPH0 = 0x83
SFR DPL1 = 0x84
SFR DPH1 = 0x85
SFR MACL = 0xE4
SFR MACH = 0xE5

ROUNDING EQU 1		; set to 0/1 to disable/enable rounding
CNS_RND_A EQU 0x0040
CNS_RND_B EQU 0x0002
;-------------------------------------------------------------------------
;- use these macros to emulate new instructions on assembler
;-------------------------------------------------------------------------

CLR_M MACRO
	DB	0xA5
	clr	A		; clr M
ENDM

ASR MACRO p
	DB	0xA5
	rr	A		; ASR	M 
ENDM
 
MAC MACRO p
	DB	0xA5
	mul	p		; mac AB
ENDM

MOV_DPTRB MACRO p
	DB	0xA5
	mov	DPTR, #p   ; mov /DPTR, #COEFF
ENDM

MOVC_ MACRO p1, p2
    DB	0xA5	 	; movc A, @/DPTR+A
	movc	A, @A+DPTR
ENDM

s_clear_fifo	SEGMENT CODE
s_store_in_fifo	SEGMENT CODE
s_compute_fir SEGMENT CODE

;-------------------------------------------------------------------------
;- DSPR (E4h)
;-
;-     <7>     <6>     <5>     <4>     <3>     <2>     <1>     <0> 
;-  +---------------------------------------------------------------+
;-  | MRW1  | MRW0  | SGMLB | SGMLA |  ---  |  XPSQ |  MVCD |  DPRB |
;-  +---------------------------------------------------------------+
;-
;- DPCF (A2h)
;-
;-     <7>     <6>     <5>     <4>     <3>     <2>     <1>     <0> 
;-  +---------------------------------------------------------------+
;-  |  NXP1 |  NXP0 |  DPD1 |  DPD0 | SIGEN |   0   |  ---  |  DPS  |
;-  +---------------------------------------------------------------+
;-
;-------------------------------------------------------------------------

;-F-----------------------------------------------------------------------
;- void _clear_fifo(int xdata *bufP, unsigned char n);
;-
;- description: clears (zeroes) the fifo
;- arguments:
;-   bufP in {R6,R7} : pointer to start of FIFO
;-   n in R5 : length of FIFO in words
;- returns:
;-   none
;-------------------------------------------------------------------------

RSEG s_clear_fifo

PUBLIC _clear_fifo

_clear_fifo:
	mov DPH, R6		; get high byte of pointer
	mov DPL, R7		; get low byte of pointer
	clr	A
clear:
	movx @DPTR, A	; clear high byte
	inc DPTR		; next byte
	movx @DPTR, A	; clear low byte
	inc DPTR		; next byte
	djnz R5, clear	; loop over entry array
	ret

;-F-----------------------------------------------------------------------
;- int xdata *_store_in_fifo(int xdata *bufP, int sample);
;-
;- description: stores one sample in the fifo
;- arguments:
;-   bufP in {R6,R7} : pointer to last (oldest) entry in FIFO
;-   sample in {R4, R5} : data to be stored
;- returns:
;-   int* in {R6,R7}
;-------------------------------------------------------------------------

RSEG s_store_in_fifo

PUBLIC _store_in_fifo

_store_in_fifo:
	push	DPCF
	mov		DPCF, #0xC0	; NXP1, NXP0 set
	mov 	DPH, R6
	mov 	DPL, R7
	mov		A, R4
	movx	@DPTR, A	; store high byte to fifo, dptr0++
	mov		A, R5
	movx	@DPTR, A	; store low byte to fifo, dptr0++
	mov		R6, DPH
	mov 	R7, DPL
	pop		DPCF
	ret
	;; return value in DPTR

;-F-----------------------------------------------------------------------
;- int _compute_fir(int xdata *bufP, int code *coeffP, unsigned char n);
;-
;- description: compute one filter output result
;- arguments:
;-   bufP in {R6,R7} : pointer to last (oldest) entry in the FIFO
;-   coeffP in {R4,R5} : pointer to coefficient table with # of shifts appended
;-   n in R3 : number of coefficients (modified)
;- returns:
;-   int in {R6,R7}
;-------------------------------------------------------------------------

RSEG s_compute_fir

PUBLIC _compute_fir

_compute_fir:
	push	DPCF
	push	DSPR

	mov 	DPH0, R6
	mov 	DPL0, R7
	mov 	DPH1, R4
	mov 	DPL1, R5

	mov		DPCF, #0xC0	; NXP1, NXP0 set
	orl		DSPR, #0x2	; MVCD  set
	CLR_M
	orl		DSPR, #1	; set dprb

LOOP:
	movx	A, @DPTR	; get high byte, dptr0++
	mov		AX, A
	movx	A, @DPTR	; get low byte,  dptr0++
	MOVC_	B, @DPTR	; movc	A,@/DPTR+A = get high byte, dptr1++
	mov		BX, B
	MOVC_	B, @DPTR	; movc	A,@/DPTR+A = get low byte, dptr1++
	MAC 	AB
	djnz	R3, LOOP

	MOVC_	B, @DPTR	; movc	A,@/DPTR+A = get # of shifts
	mov		R3, B
	mov		A, B
	jz		ROUND

SHIFT:
	ASR		M		
	djnz	R3, SHIFT

ROUND:
IF ROUNDING = 1
	mov	AX, #HIGH(CNS_RND_A)
	mov A,  #LOW(CNS_RND_A)
	mov	BX, #HIGH(CNS_RND_B)
	mov	B,  #LOW(CNS_RND_B)
	MAC AB
ENDIF
	
	; return in {R6,R7}
	mov	R6, MACH
	mov	R7, MACL
	
	pop	DSPR
	pop	DPCF
    ret

 END