/*************************************************************************
 * FILE:	iir.c
 *
 * COPYRIGHT:	(C) 2011 Atmel Corp. <www.atmel.com>
 *
 * ABSTRACT:	IIR filer example
 *
 * DESCRIPTION:	This simple example filters a stored input stream and places
 *              the output stream in RAM;
 *              NOTE: SDCC is little endian
 *
 * MODIFICATION HISTORY:
 * 2011/05/26 : created for SDCC
 *
 ************************************************************************/

#include ".\IIR.h"
#include <AT89LP6440.h>
#include <stdlib.h>

#define N_TAPS 6		// define the filter order here; must be <= 128
#define N_ASFT 4		// # of shifts required to normalize A
#define N_BSFT 12		// # of shifts required to normalize B
#define N_SAMPLES 64

__sfr __at (0xE3) FIRD;
__sfr __at (0xE2) DSPR;

#include "lib_iir.h"

#pragma callee_saves clear_fifo, store_in_fifo, compute_iir

int get_sample(unsigned char i);

/* The FIFO can only be located at 0x0000 or 0x0100 
 */
  __xdata __at (0x0000) int FIFO_A[2*(N_TAPS-1)];
  __xdata __at (0x0100) int FIFO_B[2*N_TAPS];

/* SAVE must be absolute or SDCC will overlay with FIFO
 */
  __xdata __at (0x0400) int SAVE[N_SAMPLES];

/* filter cofficients in reverse order (a5->a1,b5->b0)
 * # coeffs = N_TAPS
 */
  __code int COEFF_A[] = { 0x0a50, 0xc9ea, 0x73aa, 0x81e0, 0x4639,
 			   N_ASFT };	// <-- place # of shifts in LOW byte here 
  __code int COEFF_B[] = { 0x3df7, 0xb34d, 0x3967, 0x3967, 0xb34d, 0x3df7,
 			   N_BSFT };	// <-- place # of shifts in LOW byte here 


void main(void){

  __xdata int *fifoA_P, *fifoB_P ;
  __code int *coeffA_P, *coeffB_P ;
  int sample, result;
  unsigned char i;
	
//------------------------------------------------------------------------
// FIR Filter Setup
//------------------------------------------------------------------------

  coeffA_P = COEFF_A;
  fifoA_P = FIFO_A;
  coeffB_P = COEFF_B;
  fifoB_P = FIFO_B;

  // clear fifos 
  clear_fifo(fifoA_P, N_TAPS-1);
  clear_fifo(fifoB_P, N_TAPS);
    
  // configure MAC, FIFOs
  DSPR = 0x4C;		// set MRW0, XPSQ1, XSPQ0
 
//------------------------------------------------------------------------
// FIR Filter Computation
//------------------------------------------------------------------------

  for (i=0; i<N_SAMPLES; i++)
  { // get sample 
    sample = get_sample(i);
    // set fifo depth
    FIRD = (2*N_TAPS)-1;
    // and store in fifo
    fifoB_P = store_in_fifo(fifoB_P, sample);
    // compute intermediate 
    compute_iir_B(fifoB_P, coeffB_P, N_TAPS);
    // set fifo depth
    FIRD = (2*(N_TAPS-1))-1;
    // compute output 
    result = compute_iir_A(fifoA_P, coeffA_P, N_TAPS-1);
    // and store in fifo
    fifoA_P = store_in_fifo(fifoA_P, result);
    // and store in buffer
    SAVE[i] = result;
  }
	
  //while(1);
  __asm;
  ljmp 0xfffd
 __endasm;
}

int get_sample(unsigned char i)
{ /* in a real application this would fetch the next sample
   * from the ADC or a communication interface
   */
  __code int wave[] = { 0x0000, 0x4c7b, 0x187d, 0xe38e, 0x2d40, 0x7535, 0x3b20, 0xfec5, 
	                    0x3fff, 0x7ec4, 0x3b20, 0xf536, 0x2d40, 0x638d, 0x187e, 0xcc7c, 
	                    0xffff, 0x3383, 0xe782, 0x9c72, 0xd2be, 0x0ac9, 0xc4e0, 0x813b,
	                    0xbfff, 0x013a, 0xc4e0, 0x8aca, 0xd2be, 0x1c71, 0xe782, 0xb384,
	                    0xffff, 0x4c7b, 0x187e, 0xe38e, 0x2d40, 0x7535, 0x3b21, 0xfec5,
	                    0x3ffe, 0x7ec4, 0x3b21, 0xf536, 0x2d40, 0x638d, 0x187e, 0xcc7c,
	                    0xffff, 0x3383, 0xe783, 0x9c72, 0xd2be, 0x0ac9, 0xc4e0, 0x813b,
	                    0xbfff, 0x013a, 0xc4e0, 0x8aca, 0xd2bd, 0x1c71, 0xe783, 0xb384 };
  if (i < N_SAMPLES)
    return(wave[i]);
  else
    return(0);
}
