;-------------------------------------------------------------------------
;- FILE:	lib_iir.asm
;-
;- COPYRIGHT	(C) 2011 Atmel Corp. <www.atmel.com>
;-
;- ABSTRACT:	IIR filter and supporting routines
;-
;- DESCRIPTION: function parameters are assumed to be passed in registers
;-              NOTE: SDCC is little endian
;-
;- MODIFICATION HISTORY:
;- 2011/05/26 : created for ASX8051
;-
;-------------------------------------------------------------------------

.module lib_iir

DPCF = 0xA2
DSPR = 0xE2
FIRD = 0xE3
AX   = 0xE1
BX   = 0xF7

DPL0 = 0x82
DPH0 = 0x83
DPL1 = 0x84
DPH1 = 0x85
MACL = 0xE4
MACH = 0xE5

ROUNDING = 1		; set to 0/1 to disable/enable rounding
CNS_RND_A = 0x0080
CNS_RND_B = 0x0001

;-------------------------------------------------------------------------
;- DSPR (E4h)
;-
;-     <7>     <6>     <5>     <4>     <3>     <2>     <1>     <0> 
;-  +---------------------------------------------------------------+
;-  | MRW1  | MRW0  | SGMLB | SGMLA |  ---  |  XPSQ |  MVCD |  DPRB |
;-  +---------------------------------------------------------------+
;-
;- DPCF (A2h)
;-
;-     <7>     <6>     <5>     <4>     <3>     <2>     <1>     <0> 
;-  +---------------------------------------------------------------+
;-  |  NXP1 |  NXP0 |  DPD1 |  DPD0 | SIGEN |   0   |  ---  |  DPS  |
;-  +---------------------------------------------------------------+
;-
;-------------------------------------------------------------------------

;-F-----------------------------------------------------------------------
;- void _clear_fifo(__xdata int *bufP, unsigned char n);
;-
;- description: clears (zeroes) the fifo
;- arguments:
;-   bufP in {DPH,DPL} : pointer to start of FIFO
;-   n in DATA : length of FIFO in words
;- returns:
;-   none
;-------------------------------------------------------------------------

.globl _clear_fifo
.globl _clear_fifo_PARM_2  
        .area OSEG  
_clear_fifo_PARM_2:  
        .ds 1  
        .area CSEG   
_clear_fifo:
	push 5
	mov	R5, _clear_fifo_PARM_2
	clr	A
clear:
	movx @DPTR, A	; clear high byte
	inc DPTR		; next byte
	movx @DPTR, A	; clear low byte
	inc DPTR		; next byte
	djnz R5, clear	; loop over entry array
	pop 5
	ret

;-F-----------------------------------------------------------------------
;- __xdata int *_store_in_fifo(__xdata int *bufP, int sample);
;-
;- description: stores one sample in the fifo
;- arguments:
;-   bufP in {DPH,DPL} : pointer to last (oldest) entry in FIFO
;-   sample in DATA : data to be stored (little endian)
;- returns:
;-   int* in {DPH,DPL}
;-------------------------------------------------------------------------

.globl _store_in_fifo
.globl _store_in_fifo_PARM_2  
        .area OSEG  
_store_in_fifo_PARM_2:  
        .ds 2  
        .area CSEG   
_store_in_fifo:
	push	DPCF
	mov	DPCF, #0xC0	; NXP1, NXP0 set
	mov	A, _store_in_fifo_PARM_2
	movx	@DPTR, A	; store low byte to fifo, dptr0++
	mov	A, _store_in_fifo_PARM_2+1
	movx	@DPTR, A	; store high byte to fifo, dptr0++
	pop	DPCF
	ret
	;; return value in DPTR

;-F-----------------------------------------------------------------------
;- int _compute_iir_A(__xdata int *bufP, __code int *coeffP, unsigned char n);
;-
;- description: compute one filter output result
;- arguments:
;-   bufP in {DPH,DPL} : pointer to last (oldest) entry in the FIFO
;-   coeffP in DATA : pointer to coefficient table with # of shifts appended
;-   n in DATA : number of coefficients (modified)
;- returns:
;-   int in {DPH,DPL}
;-------------------------------------------------------------------------

.globl _compute_iir_A
.globl _compute_iir_A_PARM_2  
.globl _compute_iir_A_PARM_3  
        .area OSEG  
_compute_iir_A_PARM_2:  
        .ds 2  
_compute_iir_A_PARM_3:  
        .ds 1  
        .area CSEG
_compute_iir_A:
	push	7
	push	DPCF
	push	DSPR

	mov 	DPL1, _compute_iir_A_PARM_2
	mov 	DPH1, _compute_iir_A_PARM_2+1

	mov	DPCF, #0xC0	; NXP1, NXP0 set
	orl	DSPR, #02	; MVCD  set
	mov	R7, _compute_iir_A_PARM_3
	; don't use DPRB because little endian
	;orl	DSPR, #1	; set dprb

LOOP_A:
	.db	0xA5
	movc	A, @A+DPTR	; movc	A,@/DPTR+A = get low byte, dptr1++
	mov	B, A
	.db	0xA5
	movc	A, @A+DPTR	; movc	A,@/DPTR+A = get high byte, dptr1++
	mov	BX, A
	movx	A, @DPTR	; get low byte, dptr0++
	mov	AX, A
	movx	A, @DPTR	; get high byte,  dptr0++
	xch	A, AX
	.db	0xA5
	mul	AB		; mac AB
	djnz	R7, LOOP_A

	.db	0xA5
	movc	A, @A+DPTR	; movc	A,@/DPTR+A = get # of shifts
	mov		R7, A
	jz	ROUND_A
SHIFT_A:
	.db	0xA5
	rr	A		; ASR	M		
	djnz	R7, SHIFT_A

ROUND_A:	
.if ROUNDING = 1
	mov	A,  #<CNS_RND_A
	mov	AX, #>CNS_RND_A
	mov	B,  #<CNS_RND_B
	mov	BX, #>CNS_RND_B
	.db	0xA5
	mul	AB		; mac AB
.endif
	
	; return in DPTR
	mov	DPH, MACH
	mov	DPL, MACL
	
	pop	DSPR
	pop	DPCF
	pop	7
	ret

;-F-----------------------------------------------------------------------
;- void _compute_iir_B(__xdata int *bufP, __code int *coeffP, unsigned char n);
;-
;- description: compute one filter output result
;- arguments:
;-   bufP in {DPH,DPL} : pointer to last (oldest) entry in the FIFO
;-   coeffP in DATA : pointer to coefficient table with # of shifts appended
;-   n in DATA : number of coefficients (modified)
;- returns:
;-   none
;-------------------------------------------------------------------------

.globl _compute_iir_B
.globl _compute_iir_B_PARM_2  
.globl _compute_iir_B_PARM_3  
        .area OSEG  
_compute_iir_B_PARM_2:  
        .ds 2  
_compute_iir_B_PARM_3:  
        .ds 1  
        .area CSEG
_compute_iir_B:
	push	7
	push	DPCF
	push	DSPR

	mov 	DPL1, _compute_iir_B_PARM_2
	mov 	DPH1, _compute_iir_B_PARM_2+1

	mov	DPCF, #0xC0	; NXP1, NXP0 set
	orl	DSPR, #0x2	; MVCD  set
	mov	R7, _compute_iir_B_PARM_3
	.db	0xA5
	clr	A		; clr M
	; don't use DPRB because little endian
	;orl	DSPR, #1	; set dprb

LOOP_B:
	.db	0xA5
	movc	A, @A+DPTR	; movc	A,@/DPTR+A = get low byte, dptr1++
	mov	B, A
	.db	0xA5
	movc	A, @A+DPTR	; movc	A,@/DPTR+A = get high byte, dptr1++
	mov	BX, A
	movx	A, @DPTR	; get low byte, dptr0++
	mov	AX, A
	movx	A, @DPTR	; get high byte,  dptr0++
	xch	A, AX
	.db	0xA5
	mul	AB		; mac AB
	djnz	R7, LOOP_B

	.db	0xA5
	movc	A, @A+DPTR	; movc	A,@/DPTR+A = get # of shifts
	mov		R7, A
	dec	A
	jz	ROUND_B
	dec	R7
SHIFT_B:
	.db	0xA5
	rr	A		; ASR	M		
	djnz	R7, SHIFT_B

ROUND_B:	
.if ROUNDING = 1
	mov	A,  #1		;; round before final shift
	mov	AX, #0
	mov	B,  #1
	mov	BX, #0
	.db	0xA5
	mul	AB		; mac AB
.endif
	.db	0xA5
	rr	A		; ASR	M		
	
	pop	DSPR
	pop	DPCF
	pop	7
	ret

